<?php
use JetBrains\PhpStorm\NoReturn;

require __DIR__ . '/../cli-start.php';

cli_title('USER MAINTENANCE');

/**
 * Display the options for this utility and exit
 */
#[NoReturn]
function display_help(): void {
    printfn('Options:');
    printfn(' - add-user [e-mail] [password]');
    printfn('     Adds a new user to this instance');
    printfn(' - set-password [e-mail] [password]');
    printfn('     Sets the password for the given user');
    printfn(' - delete-user [e-mail]');
    printfn('     Deletes a user and all their data' . PHP_EOL);
    printfn('To assist with migrating from single-user to multi-user mode:');
    printfn(' - migrate-single-user [e-mail] [password]');
    printfn('     Changes the e-mail address and password for the single-user mode user');
    printfn(' - remove-single-user');
    printfn('     Removes the single-user mode user and its data');
    exit(0);
}

if ($argc < 2) display_help();

switch ($argv[1]) {
    case 'add-user':
        if ($argc < 4) {
            printfn('Missing parameters: add-user requires e-mail and password');
            exit(-1);
        }
        add_user();
        break;
    case 'set-password':
        if ($argc < 4) {
            printfn('Missing parameters: set-password requires e-mail and password');
            exit(-1);
        }
        set_password();
        break;
    case 'delete-user':
        if ($argc < 3) {
            printfn('Missing parameters: delete-user requires e-mail address');
            exit(-1);
        }
        delete_user($argv[2]);
        break;
    case 'migrate-single-user':
        if ($argc < 4) {
            printfn('Missing parameters: migrate-single-user requires e-mail and password');
            exit(-1);
        }
        migrate_single_user();
        break;
    case 'remove-single-user':
        delete_user(Security::SINGLE_USER_EMAIL);
        break;
    default:
        printfn('Unrecognized option "%s"', $argv[1]);
        display_help();
}


/**
 * Add a new user
 */
function add_user(): void {
    global $argv;
    $db = Data::getConnection();

    try {
        // Ensure there is not already a user with this e-mail address
        $user = Security::findUserByEmail($argv[2], $db);
        if ($user) {
            printfn('A user with e-mail address "%s" already exists', $argv[2]);
            return;
        }

        Security::addUser($argv[2], $argv[3], $db);

        printfn('User "%s" with password "%s" added successfully', $argv[2], $argv[3]);
    } finally {
        $db->close();
    }
}

/**
 * Set a user's password
 */
function set_password(): void {
    global $argv;

    $db = Data::getConnection();

    try {
        // Ensure this user exists
        $user = Security::findUserByEmail($argv[2], $db);
        if (!$user) {
            printfn('No user exists with e-mail address "%s"', $argv[2]);
            return;
        }

        Security::updatePassword($argv[2], $argv[3], $db);

        printfn('User "%s" password set to "%s" successfully', $argv[2], $argv[3]);
    } finally {
        $db->close();
    }
}

/**
 * Delete a user
 *
 * @param string $email The e-mail address of the user to be deleted
 */
function delete_user(string $email): void {

    $db = Data::getConnection();

    try {
        $displayUser = $email == Security::SINGLE_USER_EMAIL ? 'single-user mode user' : "user \"$email\"";

        // Get the ID for the provided e-mail address
        $user = Security::findUserByEmail($email, $db);
        if (!$user) {
            printfn('No %s exists', $displayUser);
            return;
        }

        $feedCountQuery = $db->prepare('SELECT COUNT(*) FROM feed WHERE user_id = :user');
        $feedCountQuery->bindValue(':user', $user['id']);
        $feedCountResult = $feedCountQuery->execute();
        if (!$feedCountResult) {
            printfn('SQLite error: %s', $db->lastErrorMsg());
            return;
        }
        $feedCount = $feedCountResult->fetchArray(SQLITE3_NUM);

        $proceed = readline("Delete the $displayUser and their $feedCount[0] feed(s)? (y/N)" . PHP_EOL);
        if (!$proceed || !str_starts_with(strtolower($proceed), 'y')) {
            printfn('Deletion canceled');
            return;
        }

        $itemDelete = $db->prepare('DELETE FROM item WHERE feed_id IN (SELECT id FROM feed WHERE user_id = :user)');
        $itemDelete->bindValue(':user', $user['id']);
        $itemDelete->execute();

        $feedDelete = $db->prepare('DELETE FROM feed WHERE user_id = :user');
        $feedDelete->bindValue(':user', $user['id']);
        $feedDelete->execute();

        $userDelete = $db->prepare('DELETE FROM frc_user WHERE id = :user');
        $userDelete->bindValue(':user', $user['id']);
        $userDelete->execute();

        printfn(strtoupper(substr($displayUser, 0, 1)) . substr($displayUser, 1) . ' deleted successfully');
    } finally {
        $db->close();
    }
}

/**
 * Change the single-user mode user to a different e-mail address and password
 */
function migrate_single_user(): void {
    global $argv;

    $db = Data::getConnection();

    try {
        $single = Security::findUserByEmail(Security::SINGLE_USER_EMAIL, $db);
        if (!$single) {
            printfn('There is no single-user mode user to be migrated');
            return;
        }

        $migrateQuery = $db->prepare('UPDATE frc_user SET email = :email, password = :password WHERE id = :id');
        $migrateQuery->bindValue(':email',    $argv[2]);
        $migrateQuery->bindValue(':password', password_hash($argv[3], Security::PW_ALGORITHM));
        $migrateQuery->bindValue(':id',       $single['id']);
        $migrateQuery->execute();

        printfn('The single user has been moved to "%s", with password "%s"', $argv[2], $argv[3]);
    } finally {
        $db->close();
    }
}
